/** @file
 *
 * @defgroup hal_intpin Intpin Handler
 * @{
 * @ingroup app_common
 *
 * @brief Intpins handling module.
 *
 * @details The intpin handler uses the @ref app_gpiote to detect that a intpin has been
 *          triggered. 
 *          Use the USE_SCHEDULER parameter of the APP_BUTTON_INIT() macro to select if the
 *          @ref app_scheduler is to be used or not.
 *
 * @note    Even if the scheduler is not used, hal_intpin.h will include app_scheduler.h, so when
 *          compiling, app_scheduler.h must be available in one of the compiler include paths.
 */

#ifndef HAL_INTPIN_H
#define HAL_INTPIN_H

#include <stdint.h>
#include <stdbool.h>
#include "nrf.h"
#include "app_error.h"
#include "app_scheduler.h"
#include "nrf_gpio.h"

#define HAL_INTPIN_SCHED_EVT_SIZE sizeof(hal_intpin_event_t)   /**< Size of intpin events being passed through the scheduler (is to be used for computing the maximum size of scheduler events). */
#define HAL_INTPIN_EDGE_HI2LO  0x02                            /**< Indicates that a intpin is active high. */
#define HAL_INTPIN_EDGE_LO2HI  0x01                            /**< Indicates that a intpin is active low. */
#define HAL_INTPIN_EDGE_BOTH   0x03

/**@brief Intpin event handler type. */
typedef void (*hal_intpin_handler_t)(uint8_t pin_no, uint8_t intpin_action);

/**@brief Type of function for passing events from the Intpin Handler module to the scheduler. */
typedef uint32_t (*hal_intpin_evt_schedule_func_t) (hal_intpin_handler_t intpin_handler,
                                                            uint8_t              pin_no,
                                                            uint8_t              intpin_action);

/**@brief Intpin configuration structure. */
typedef struct
{
    uint8_t              pin_no;           /**< Pin to be used as a intpin. */
    uint8_t              trigger_edge;     /**< APP_BUTTON_ACTIVE_HIGH or APP_BUTTON_ACTIVE_LOW. */
    nrf_gpio_pin_pull_t  enable_pull_cfg;  /**< Pull-up or -down configuration. */
    nrf_gpio_pin_pull_t  disable_pull_cfg;
    hal_intpin_handler_t intpin_handler;   /**< Handler to be called when intpin is pushed. */
} hal_intpin_cfg_t;

/**@brief Macro for initializing the Intpin Handler module.
 *
 * @details It will initialize the specified pins as intpins, and configure the Intpin Handler
 *          module as a GPIOTE user (but it will not enable intpin detection). It will also connect
 *          the Intpin Handler module to the scheduler (if specified).
 *
 * @param[in]  BUTTONS           Array of intpins to be used (type hal_intpin_cfg_t, must be
 *                               static!).
 * @param[in]  BUTTON_COUNT      Number of intpins.
 * @param[in]  USE_SCHEDULER     TRUE if the application is using the event scheduler,
 *                               FALSE otherwise.
 */
#define HAL_INTPIN_INIT(INTPINS, INTPIN_COUNT)                                      \
    do                                                                              \
    {                                                                               \
        uint32_t ERR_CODE = hal_intpin_init((INTPINS), (INTPIN_COUNT));             \
        APP_ERROR_CHECK(ERR_CODE);                                                  \
    } while (0)

/**@brief Function for initializing the Intpins.
 *
 * @details This function will initialize the specified pins as intpins, and configure the Intpin
 *          Handler module as a GPIOTE user (but it will not enable intpin detection).
 *
 * @note Normally initialization should be done using the APP_BUTTON_INIT() macro, as that will take
 *       care of connecting the Intpins module to the scheduler (if specified).
 *
 * @note hal_intpin_enable() function must be called in order to enable the intpin detection.    
 *
 * @param[in]  p_intpins           Array of intpins to be used (NOTE: Must be static!).
 * @param[in]  intpin_count        Number of intpins.
 * @param[in]  detection_delay     Delay from a GPIOTE event until a intpin is reported as pushed.
 * @param[in]  evt_schedule_func   Function for passing intpin events to the scheduler. Point to
 *                                 hal_intpin_evt_schedule() to connect to the scheduler. Set to
 *                                 NULL to make the Intpins module call the event handler directly
 *                                 from the delayed intpin push detection timeout handler.
 *
 * @return   NRF_SUCCESS on success, otherwise an error code.
 */
uint32_t hal_intpin_init(hal_intpin_cfg_t *             p_intpins,
                         uint8_t                        intpin_count);

/**@brief Function for enabling intpin detection.
 *
 * @retval  NRF_ERROR_INVALID_PARAM   GPIOTE has to many users.
 * @retval  NRF_ERROR_INVALID_STATE   Intpin or GPIOTE not initialized.
 * @retval  NRF_SUCCESS               Intpin detection successfully enabled.
 */
uint32_t hal_intpin_enable(void);

/**@brief Function for disabling intpin detection.
 *
 * @retval  NRF_ERROR_INVALID_PARAM   GPIOTE has to many users.
 * @retval  NRF_ERROR_INVALID_STATE   Intpin or GPIOTE not initialized.
 * @retval  NRF_SUCCESS               Intpin detection successfully enabled.
 */
uint32_t hal_intpin_disable(void);

void hal_intpin_enable_pin(uint32_t pin);

void hal_intpin_disable_pin(uint32_t pin);

#endif // HAL_INTPIN_H

/** @} */
